@@ -1,6 +1,4 @@
-// $Id: PolicyFile.cc 1473 2005-10-06 21:32:45Z vern $
-
-#include "config.h"
+#include "bro-config.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -8,6 +6,7 @@
 
 #include <map>
 #include <stdio.h>
+#include <errno.h>
 #include <string>
 #include <vector>
 
@@ -16,6 +15,7 @@ using namespace std;
 #include "Debug.h"
 #include "util.h"
 #include "PolicyFile.h"
+#include "Reporter.h"
 
 struct PolicyFile {
 	PolicyFile ()	{ filedata = 0; lmtime = 0; }
@@ -32,7 +32,7 @@ static PolicyFileMap policy_files;
 int how_many_lines_in(const char* policy_filename)
 	{
 	if ( ! policy_filename )
-		internal_error("NULL value passed to how_many_lines_in\n");
+		reporter->InternalError("NULL value passed to how_many_lines_in\n");
 
 	FILE* throwaway = fopen(policy_filename, "r");
 	if ( ! throwaway )
@@ -81,7 +81,14 @@ bool LoadPolicyFileText(const char* policy_filename)
 	policy_files.insert(PolicyFileMap::value_type(policy_filename, pf));
 
 	struct stat st;
-	fstat(fileno(f), &st);
+	if ( fstat(fileno(f), &st) != 0 )
+		{
+		char buf[256];
+		bro_strerror_r(errno, buf, sizeof(buf));
+		reporter->Error("fstat failed on %s: %s", policy_filename, buf);
+		fclose(f);
+		return false;
+		}
 
 	pf->lmtime = st.st_mtime;
 	off_t size = st.st_size;
@@ -89,7 +96,8 @@ bool LoadPolicyFileText(const char* policy_filename)
 	// ### This code is not necessarily Unicode safe!
 	// (probably fine with UTF-8)
 	pf->filedata = new char[size+1];
-	fread(pf->filedata, size, 1, f);
+	if ( fread(pf->filedata, size, 1, f) != 1 )
+        reporter->InternalError("Failed to fread() file data");
 	pf->filedata[size] = 0;
 	fclose(f);
 
