@@ -1,6 +1,6 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
-#include "config.h"
+#include "bro-config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -12,23 +12,17 @@
 #include <getopt.h>
 #endif
 
-#ifdef USE_CURL
-#include <curl/curl.h>
-#endif
-
 #ifdef USE_IDMEF
 extern "C" {
 #include <libidmef/idmefxml.h>
 }
 #endif
 
-#include <openssl/md5.h>
-
-extern "C" void OPENSSL_add_all_algorithms_conf(void);
+#include <openssl/ssl.h>
+#include <openssl/err.h>
 
 #include "bsd-getopt-long.h"
 #include "input.h"
-#include "ScriptAnaly.h"
 #include "DNS_Mgr.h"
 #include "Frame.h"
 #include "Scope.h"
@@ -50,17 +44,20 @@ extern "C" void OPENSSL_add_all_algorithms_conf(void);
 #include "EventRegistry.h"
 #include "Stats.h"
 #include "Brofiler.h"
+#include "Traverse.h"
 
 #include "threading/Manager.h"
 #include "input/Manager.h"
 #include "logging/Manager.h"
-#include "logging/writers/Ascii.h"
-#include "input/readers/Raw.h"
+#include "logging/writers/ascii/Ascii.h"
+#include "input/readers/raw/Raw.h"
 #include "analyzer/Manager.h"
 #include "analyzer/Tag.h"
 #include "plugin/Manager.h"
 #include "file_analysis/Manager.h"
 #include "broxygen/Manager.h"
+#include "iosource/Manager.h"
+#include "broker/Manager.h"
 
 #include "binpac_bro.h"
 
@@ -84,11 +81,9 @@ int perftools_leaks = 0;
 int perftools_profile = 0;
 #endif
 
-const char* prog;
-char* writefile = 0;
-name_list prefixes;
 DNS_Mgr* dns_mgr;
 TimerMgr* timer_mgr;
+PortManager* port_mgr = 0;
 logging::Manager* log_mgr = 0;
 threading::Manager* thread_mgr = 0;
 input::Manager* input_mgr = 0;
@@ -96,6 +91,12 @@ plugin::Manager* plugin_mgr = 0;
 analyzer::Manager* analyzer_mgr = 0;
 file_analysis::Manager* file_mgr = 0;
 broxygen::Manager* broxygen_mgr = 0;
+iosource::Manager* iosource_mgr = 0;
+bro_broker::Manager* broker_mgr = 0;
+
+const char* prog;
+char* writefile = 0;
+name_list prefixes;
 Stmt* stmts;
 EventHandlerPtr net_done = 0;
 RuleMatcher* rule_matcher = 0;
@@ -109,15 +110,12 @@ ProfileLogger* profiling_logger = 0;
 ProfileLogger* segment_logger = 0;
 SampleLogger* sample_logger = 0;
 int signal_val = 0;
-int optimize = 0;
-int do_notice_analysis = 0;
-int rule_bench = 0;
-SecondaryPath* secondary_path = 0;
 extern char version[];
 char* command_line_policy = 0;
 vector<string> params;
+set<string> requested_plugins;
 char* proc_status_file = 0;
-int snaplen = 0;	// this gets set from the scripting-layer's value
+int old_comm_usage_count = 0;
 
 OpaqueType* md5_type = 0;
 OpaqueType* sha1_type = 0;
@@ -127,6 +125,7 @@ OpaqueType* cardinality_type = 0;
 OpaqueType* topk_type = 0;
 OpaqueType* bloomfilter_type = 0;
 OpaqueType* x509_opaque_type = 0;
+OpaqueType* ocsp_resp_opaque_type = 0;
 
 // Keep copy of command line
 int bro_argc;
@@ -158,7 +157,7 @@ const char* bro_dns_fake()
 		return "on";
 	}
 
-void usage()
+void usage(int code = 1)
 	{
 	fprintf(stderr, "bro version %s\n", bro_version());
 	fprintf(stderr, "usage: %s [options] [file ...]\n", prog);
@@ -169,53 +168,46 @@ void usage()
 	fprintf(stderr, "    -e|--exec <bro code>           | augment loaded policies by given code\n");
 	fprintf(stderr, "    -f|--filter <filter>           | tcpdump filter\n");
 	fprintf(stderr, "    -g|--dump-config               | dump current config into .state dir\n");
-	fprintf(stderr, "    -h|--help|-?                   | command line help\n");
+	fprintf(stderr, "    -h|--help                      | command line help\n");
 	fprintf(stderr, "    -i|--iface <interface>         | read from given interface\n");
 	fprintf(stderr, "    -p|--prefix <prefix>           | add given prefix to policy file resolution\n");
 	fprintf(stderr, "    -r|--readfile <readfile>       | read from given tcpdump file\n");
-	fprintf(stderr, "    -y|--flowfile <file>[=<ident>] | read from given flow file\n");
-	fprintf(stderr, "    -Y|--netflow <ip>:<prt>[=<id>] | read flow from socket\n");
 	fprintf(stderr, "    -s|--rulefile <rulefile>       | read rules from given file\n");
 	fprintf(stderr, "    -t|--tracefile <tracefile>     | activate execution tracing\n");
-	fprintf(stderr, "    -w|--writefile <writefile>     | write to given tcpdump file\n");
 	fprintf(stderr, "    -v|--version                   | print version and exit\n");
+	fprintf(stderr, "    -w|--writefile <writefile>     | write to given tcpdump file\n");
 	fprintf(stderr, "    -x|--print-state <file.bst>    | print contents of state file\n");
-	fprintf(stderr, "    -z|--analyze <analysis>        | run the specified policy file analysis\n");
 #ifdef DEBUG
-	fprintf(stderr, "    -B|--debug <dbgstreams>        | Enable debugging output for selected streams\n");
+	fprintf(stderr, "    -B|--debug <dbgstreams>        | Enable debugging output for selected streams ('-B help' for help)\n");
 #endif
 	fprintf(stderr, "    -C|--no-checksums              | ignore checksums\n");
-	fprintf(stderr, "    -D|--dfa-size <size>           | DFA state cache size\n");
 	fprintf(stderr, "    -F|--force-dns                 | force DNS\n");
+	fprintf(stderr, "    -G|--load-seeds <file>         | load seeds from given file\n");
+	fprintf(stderr, "    -H|--save-seeds <file>         | save seeds to given file\n");
 	fprintf(stderr, "    -I|--print-id <ID name>        | print out given ID\n");
-	fprintf(stderr, "    -K|--md5-hashkey <hashkey>     | set key for MD5-keyed hashing\n");
-	fprintf(stderr, "    -L|--rule-benchmark            | benchmark for rules\n");
 	fprintf(stderr, "    -N|--print-plugins             | print available plugins and exit (-NN for verbose)\n");
-	fprintf(stderr, "    -O|--optimize                  | optimize policy script\n");
 	fprintf(stderr, "    -P|--prime-dns                 | prime DNS\n");
+	fprintf(stderr, "    -Q|--time                      | print execution time summary to stderr\n");
 	fprintf(stderr, "    -R|--replay <events.bst>       | replay events\n");
 	fprintf(stderr, "    -S|--debug-rules               | enable rule debugging\n");
 	fprintf(stderr, "    -T|--re-level <level>          | set 'RE_level' for rules\n");
 	fprintf(stderr, "    -U|--status-file <file>        | Record process status in file\n");
 	fprintf(stderr, "    -W|--watchdog                  | activate watchdog timer\n");
-	fprintf(stderr, "    -X|--broxygen                  | generate documentation based on config file\n");
+	fprintf(stderr, "    -X|--broxygen <cfgfile>        | generate documentation based on config file\n");
 
 #ifdef USE_PERFTOOLS_DEBUG
 	fprintf(stderr, "    -m|--mem-leaks                 | show leaks  [perftools]\n");
 	fprintf(stderr, "    -M|--mem-profile               | record heap [perftools]\n");
-#endif
-#if 0 // Broken
-	fprintf(stderr, "    -X <file.bst>                  | print contents of state file as XML\n");
 #endif
 	fprintf(stderr, "    --pseudo-realtime[=<speedup>]  | enable pseudo-realtime for performance evaluation (default 1)\n");
-	fprintf(stderr, "    --load-seeds <file>            | load seeds from given file\n");
-	fprintf(stderr, "    --save-seeds <file>            | save seeds to given file\n");
 
 #ifdef USE_IDMEF
 	fprintf(stderr, "    -n|--idmef-dtd <idmef-msg.dtd> | specify path to IDMEF DTD file\n");
 #endif
 
-	fprintf(stderr, "    $BROPATH                       | file search path (%s)\n", bro_path());
+	fprintf(stderr, "    $BROPATH                       | file search path (%s)\n", bro_path().c_str());
+	fprintf(stderr, "    $BRO_PLUGIN_PATH               | plugin search path (%s)\n", bro_plugin_path());
+	fprintf(stderr, "    $BRO_PLUGIN_ACTIVATE           | plugins to always activate (%s)\n", bro_plugin_activate());
 	fprintf(stderr, "    $BRO_PREFIXES                  | prefix list (%s)\n", bro_prefixes().c_str());
 	fprintf(stderr, "    $BRO_DNS_FAKE                  | disable DNS lookups (%s)\n", bro_dns_fake());
 	fprintf(stderr, "    $BRO_SEED_FILE                 | file to load seeds from (not set)\n");
@@ -224,37 +216,18 @@ void usage()
 	fprintf(stderr, "    $BRO_DISABLE_BROXYGEN          | Disable Broxygen documentation support (%s)\n", getenv("BRO_DISABLE_BROXYGEN") ? "set" : "not set");
 
 	fprintf(stderr, "\n");
-	fprintf(stderr, "    Supported log formats: ");
 
-	bool first = true;
-	list<string> fmts = logging::Manager::SupportedFormats();
-
-	for ( list<string>::const_iterator i = fmts.begin(); i != fmts.end(); ++i )
-		{
-		if ( *i == "None" )
-			// Skip, it's uninteresting.
-			continue;
-
-		if ( ! first )
-			fprintf(stderr, ",");
-
-		fprintf(stderr, "%s", (*i).c_str());
-		first = false;
-		}
-
-	fprintf(stderr, "\n");
-
-	exit(1);
+	exit(code);
 	}
 
-void show_plugins(int level)
+bool show_plugins(int level)
 	{
-	plugin::Manager::plugin_list plugins = plugin_mgr->Plugins();
+	plugin::Manager::plugin_list plugins = plugin_mgr->ActivePlugins();
 
 	if ( ! plugins.size() )
 		{
 		printf("No plugins registered, not even any built-ins. This is probably a bug.\n");
-		return;
+		return false;
 		}
 
 	ODesc d;
@@ -262,15 +235,39 @@ void show_plugins(int level)
 	if ( level == 1 )
 		d.SetShort();
 
+	int count = 0;
+
 	for ( plugin::Manager::plugin_list::const_iterator i = plugins.begin(); i != plugins.end(); i++ )
 		{
+		if ( requested_plugins.size()
+		     && requested_plugins.find((*i)->Name()) == requested_plugins.end() )
+			continue;
+
 		(*i)->Describe(&d);
 
 		if ( ! d.IsShort() )
 			d.Add("\n");
+
+		++count;
 		}
 
 	printf("%s", d.Description());
+
+	plugin::Manager::inactive_plugin_list inactives = plugin_mgr->InactivePlugins();
+
+	if ( inactives.size() && ! requested_plugins.size() )
+		{
+		printf("\nInactive dynamic plugins:\n");
+
+		for ( plugin::Manager::inactive_plugin_list::const_iterator i = inactives.begin(); i != inactives.end(); i++ )
+			{
+			string name = (*i).first;
+			string path = (*i).second;
+			printf("  %s (%s)\n", name.c_str(), path.c_str());
+			}
+		}
+
+	return count != 0;
 	}
 
 void done_with_network()
@@ -335,6 +332,10 @@ void terminate_bro()
 
 	terminating = true;
 
+	// File analysis termination may produce events, so do it early on in
+	// the termination process.
+	file_mgr->Terminate();
+
 	brofiler.WriteStats();
 
 	EventHandlerPtr bro_done = internal_handler("bro_done");
@@ -359,10 +360,10 @@ void terminate_bro()
 
 	mgr.Drain();
 
-	file_mgr->Terminate();
 	log_mgr->Terminate();
 	input_mgr->Terminate();
 	thread_mgr->Terminate();
+	broker_mgr->Terminate();
 
 	mgr.Drain();
 
@@ -370,20 +371,18 @@ void terminate_bro()
 
 	delete broxygen_mgr;
 	delete timer_mgr;
-	delete dns_mgr;
 	delete persistence_serializer;
-	delete event_player;
 	delete event_serializer;
 	delete state_serializer;
 	delete event_registry;
-	delete secondary_path;
-	delete remote_serializer;
 	delete analyzer_mgr;
 	delete file_mgr;
 	delete log_mgr;
 	delete plugin_mgr;
-	delete thread_mgr;
 	delete reporter;
+	// broker_mgr is deleted via iosource_mgr
+	delete iosource_mgr;
+	delete port_mgr;
 
 	reporter = 0;
 	}
@@ -427,10 +426,76 @@ static void bro_new_handler()
 	out_of_memory("new");
 	}
 
+static auto old_comm_ids = std::set<const char*, CompareString>{
+	"connect",
+	"disconnect",
+	"request_remote_events",
+	"request_remote_sync",
+	"request_remote_logs",
+	"set_accept_state",
+	"set_compression_level",
+	"listen",
+	"send_id",
+	"terminate_communication",
+	"complete_handshake",
+	"send_ping",
+	"send_current_packet",
+	"get_event_peer",
+	"send_capture_filter",
+	"suspend_state_updates",
+	"resume_state_updates",
+};
+
+static bool is_old_comm_usage(const ID* id)
+	{
+	auto name = id->Name();
+
+	if ( old_comm_ids.find(name) == old_comm_ids.end() )
+		return false;
+
+	return true;
+	}
+
+class OldCommUsageTraversalCallback : public TraversalCallback {
+public:
+	virtual TraversalCode PreExpr(const Expr* expr) override
+		{
+		switch ( expr->Tag() ) {
+		case EXPR_CALL:
+			{
+			const CallExpr* call = static_cast<const CallExpr*>(expr);
+			auto func = call->Func();
+
+			if ( func->Tag() == EXPR_NAME )
+				{
+				const NameExpr* ne = static_cast<const NameExpr*>(func);
+				auto id = ne->Id();
+
+				if ( is_old_comm_usage(id) )
+					++old_comm_usage_count;
+				}
+			}
+			break;
+		default:
+			break;
+		}
+
+		return TC_CONTINUE;
+		}
+};
+
+static void find_old_comm_usages()
+	{
+	OldCommUsageTraversalCallback cb;
+	traverse_all(&cb);
+	}
+
 int main(int argc, char** argv)
 	{
 	std::set_new_handler(bro_new_handler);
 
+	double time_start = current_time(true);
+
 	brofiler.ReadStats();
 
 	bro_argc = argc;
@@ -441,8 +506,6 @@ int main(int argc, char** argv)
 
 	name_list interfaces;
 	name_list read_files;
-	name_list netflows;
-	name_list flow_files;
 	name_list rule_files;
 	char* bst_file = 0;
 	char* id_name = 0;
@@ -453,14 +516,13 @@ int main(int argc, char** argv)
 	char* debug_streams = 0;
 	int parse_only = false;
 	int bare_mode = false;
-	int seed = 0;
 	int dump_cfg = false;
-	int to_xml = 0;
 	int do_watchdog = 0;
 	int override_ignore_checksums = 0;
 	int rule_debug = 0;
 	int RE_level = 4;
 	int print_plugins = 0;
+	int time_bro = 0;
 
 	static struct option long_opts[] = {
 		{"parse-only",	no_argument,		0,	'a'},
@@ -474,28 +536,21 @@ int main(int argc, char** argv)
 		{"broxygen",		required_argument,		0,	'X'},
 		{"prefix",		required_argument,	0,	'p'},
 		{"readfile",		required_argument,	0,	'r'},
-		{"flowfile",		required_argument,	0,	'y'},
-		{"netflow",		required_argument,	0,	'Y'},
 		{"rulefile",		required_argument,	0,	's'},
 		{"tracefile",		required_argument,	0,	't'},
 		{"writefile",		required_argument,	0,	'w'},
 		{"version",		no_argument,		0,	'v'},
 		{"print-state",		required_argument,	0,	'x'},
-		{"analyze",		required_argument,	0,	'z'},
 		{"no-checksums",	no_argument,		0,	'C'},
-		{"dfa-cache",		required_argument,	0,	'D'},
 		{"force-dns",		no_argument,		0,	'F'},
 		{"load-seeds",		required_argument,	0,	'G'},
 		{"save-seeds",		required_argument,	0,	'H'},
-		{"set-seed",		required_argument,	0,	'J'},
-		{"md5-hashkey",		required_argument,	0,	'K'},
-		{"rule-benchmark",	no_argument,		0,	'L'},
 		{"print-plugins",	no_argument,		0,	'N'},
-		{"optimize",		no_argument,		0,	'O'},
 		{"prime-dns",		no_argument,		0,	'P'},
+		{"time",		no_argument,		0,	'Q'},
 		{"replay",		required_argument,	0,	'R'},
 		{"debug-rules",		no_argument,		0,	'S'},
-		{"re-level",		required_argument,	0,	'R'},
+		{"re-level",		required_argument,	0,	'T'},
 		{"watchdog",		no_argument,		0,	'W'},
 		{"print-id",		required_argument,	0,	'I'},
 		{"status-file",		required_argument,	0,	'U'},
@@ -543,7 +598,7 @@ int main(int argc, char** argv)
 	opterr = 0;
 
 	char opts[256];
-	safe_strncpy(opts, "B:D:e:f:I:i:K:l:n:p:R:r:s:T:t:U:w:x:X:y:Y:z:CFGLNOPSWabdghv",
+	safe_strncpy(opts, "B:e:f:G:H:I:i:n:p:R:r:s:T:t:U:w:x:X:CFNPQSWabdghv",
 		     sizeof(opts));
 
 #ifdef USE_PERFTOOLS_DEBUG
@@ -578,6 +633,10 @@ int main(int argc, char** argv)
 			dump_cfg = true;
 			break;
 
+		case 'h':
+			usage(0);
+			break;
+
 		case 'i':
 			interfaces.append(optarg);
 			break;
@@ -599,32 +658,27 @@ int main(int argc, char** argv)
 			g_trace_state.TraceOn();
 			break;
 
+		case 'v':
+			fprintf(stdout, "%s version %s\n", prog, bro_version());
+			exit(0);
+			break;
+
 		case 'w':
 			writefile = optarg;
 			break;
 
-		case 'y':
-			flow_files.append(optarg);
+		case 'x':
+			bst_file = optarg;
 			break;
 
-		case 'z':
-			if ( streq(optarg, "notice") )
-				do_notice_analysis = 1;
-			else
-				{
-				fprintf(stderr, "Unknown analysis type: %s\n", optarg);
-				exit(1);
-				}
+		case 'B':
+			debug_streams = optarg;
 			break;
 
 		case 'C':
 			override_ignore_checksums = 1;
 			break;
 
-		case 'D':
-			dfa_state_cache_size = atoi(optarg);
-			break;
-
 		case 'E':
 			pseudo_realtime = 1.0;
 			if ( optarg )
@@ -633,7 +687,7 @@ int main(int argc, char** argv)
 
 		case 'F':
 			if ( dns_type != DNS_DEFAULT )
-				usage();
+				usage(1);
 			dns_type = DNS_FORCE;
 			break;
 
@@ -649,33 +703,20 @@ int main(int argc, char** argv)
 			id_name = optarg;
 			break;
 
-		case 'J':
-			seed = atoi(optarg);
-			break;
-
-		case 'K':
-			MD5((const u_char*) optarg, strlen(optarg), shared_hmac_md5_key);
-			hmac_key_set = 1;
-			break;
-
-		case 'L':
-			++rule_bench;
-			break;
-
 		case 'N':
 			++print_plugins;
 			break;
 
-		case 'O':
-			optimize = 1;
-			break;
-
 		case 'P':
 			if ( dns_type != DNS_DEFAULT )
-				usage();
+				usage(1);
 			dns_type = DNS_PRIME;
 			break;
 
+		case 'Q':
+			time_bro = 1;
+			break;
+
 		case 'R':
 			events_file = optarg;
 			break;
@@ -696,17 +737,8 @@ int main(int argc, char** argv)
 			do_watchdog = 1;
 			break;
 
-		case 'Y':
-			netflows.append(optarg);
-			break;
-
-		case 'h':
-			usage();
-			break;
-
-		case 'v':
-			fprintf(stderr, "%s version %s\n", prog, bro_version());
-			exit(0);
+		case 'X':
+			broxygen_config = optarg;
 			break;
 
 #ifdef USE_PERFTOOLS_DEBUG
@@ -719,20 +751,6 @@ int main(int argc, char** argv)
 			break;
 #endif
 
-		case 'x':
-			bst_file = optarg;
-			break;
-#if 0 // broken
-		case 'X':
-			bst_file = optarg;
-			to_xml = 1;
-			break;
-#endif
-
-		case 'X':
-			broxygen_config = optarg;
-			break;
-
 #ifdef USE_IDMEF
 		case 'n':
 			fprintf(stderr, "Using IDMEF XML DTD from %s\n", optarg);
@@ -740,10 +758,6 @@ int main(int argc, char** argv)
 			break;
 #endif
 
-		case 'B':
-			debug_streams = optarg;
-			break;
-
 		case 0:
 			// This happens for long options that don't have
 			// a short-option equivalent.
@@ -751,7 +765,7 @@ int main(int argc, char** argv)
 
 		case '?':
 		default:
-			usage();
+			usage(1);
 			break;
 		}
 
@@ -760,30 +774,28 @@ int main(int argc, char** argv)
 
 	bro_start_time = current_time(true);
 
+	port_mgr = new PortManager();
 	reporter = new Reporter();
 	thread_mgr = new threading::Manager();
+	plugin_mgr = new plugin::Manager();
 
 #ifdef DEBUG
 	if ( debug_streams )
+		{
 		debug_logger.EnableStreams(debug_streams);
+		debug_logger.OpenDebugLog("debug");
+		}
 #endif
 
-	init_random_seed(seed, (seed_load_file && *seed_load_file ? seed_load_file : 0) , seed_save_file);
+	init_random_seed((seed_load_file && *seed_load_file ? seed_load_file : 0) , seed_save_file);
 	// DEBUG_MSG("HMAC key: %s\n", md5_digest_print(shared_hmac_md5_key));
 	init_hash_function();
 
-	// Must come after hash initialization.
-	binpac::init();
-
 	ERR_load_crypto_strings();
 	OPENSSL_add_all_algorithms_conf();
 	SSL_library_init();
 	SSL_load_error_strings();
 
-#ifdef USE_CURL
-	curl_global_init(CURL_GLOBAL_ALL);
-#endif
-
 	int r = sqlite3_initialize();
 
 	if ( r != SQLITE_OK )
@@ -793,9 +805,8 @@ int main(int argc, char** argv)
 	// seed the PRNG. We should do this here (but at least Linux, FreeBSD
 	// and Solaris provide /dev/urandom).
 
-	if ( (interfaces.length() > 0 || netflows.length() > 0) &&
-	     (read_files.length() > 0 || flow_files.length() > 0 ))
-		usage();
+	if ( interfaces.length() > 0 && read_files.length() > 0 )
+		usage(1);
 
 #ifdef USE_IDMEF
 	char* libidmef_dtd_path_cstr = new char[libidmef_dtd_path.length() + 1];
@@ -810,28 +821,34 @@ int main(int argc, char** argv)
 
 	broxygen_mgr = new broxygen::Manager(broxygen_config, bro_argv[0]);
 
-	add_input_file("base/init-bare.bro");
+	add_essential_input_file("base/init-bare.bro");
+	add_essential_input_file("base/init-frameworks-and-bifs.bro");
+
 	if ( ! bare_mode )
 		add_input_file("base/init-default.bro");
 
+	plugin_mgr->SearchDynamicPlugins(bro_plugin_path());
+
 	if ( optind == argc &&
-	     read_files.length() == 0 && flow_files.length() == 0 &&
+	     read_files.length() == 0 &&
 	     interfaces.length() == 0 &&
 	     ! (id_name || bst_file) && ! command_line_policy && ! print_plugins )
 		add_input_file("-");
 
-	// Process remaining arguments.  X=Y arguments indicate script
-	// variable/parameter assignments.  The remainder are treated
-	// as scripts to load.
+	// Process remaining arguments. X=Y arguments indicate script
+	// variable/parameter assignments. X::Y arguments indicate plugins to
+	// activate/query. The remainder are treated as scripts to load.
 	while ( optind < argc )
 		{
 		if ( strchr(argv[optind], '=') )
 			params.push_back(argv[optind++]);
+		else if ( strstr(argv[optind], "::") )
+			requested_plugins.insert(argv[optind++]);
 		else
 			add_input_file(argv[optind++]);
 		}
 
-	push_scope(0);
+	push_scope(nullptr, nullptr);
 
 	dns_mgr = new DNS_Mgr(dns_type);
 
@@ -840,26 +857,43 @@ int main(int argc, char** argv)
 	// policy, but we can't parse policy without DNS resolution.
 	dns_mgr->SetDir(".state");
 
+	iosource_mgr = new iosource::Manager();
 	persistence_serializer = new PersistenceSerializer();
 	remote_serializer = new RemoteSerializer();
 	event_registry = new EventRegistry();
 	analyzer_mgr = new analyzer::Manager();
 	log_mgr = new logging::Manager();
 	input_mgr = new input::Manager();
-	plugin_mgr = new plugin::Manager();
 	file_mgr = new file_analysis::Manager();
+	broker_mgr = new bro_broker::Manager(read_files.length() > 0);
 
 	plugin_mgr->InitPreScript();
 	analyzer_mgr->InitPreScript();
 	file_mgr->InitPreScript();
 	broxygen_mgr->InitPreScript();
 
+	bool missing_plugin = false;
+
+	for ( set<string>::const_iterator i = requested_plugins.begin();
+	      i != requested_plugins.end(); i++ )
+		{
+		if ( ! plugin_mgr->ActivateDynamicPlugin(*i) )
+			missing_plugin = true;
+		}
+
+	if ( missing_plugin )
+		reporter->FatalError("Failed to activate requested dynamic plugin(s).");
+
+	plugin_mgr->ActivateDynamicPlugins(! bare_mode);
+
 	if ( events_file )
 		event_player = new EventPlayer(events_file);
 
-	init_event_handlers();
+	// Must come after plugin activation (and also after hash
+	// initialization).
+	binpac::init();
 
-	input::reader::Raw::ClassInit();
+	init_event_handlers();
 
 	md5_type = new OpaqueType("md5");
 	sha1_type = new OpaqueType("sha1");
@@ -869,6 +903,7 @@ int main(int argc, char** argv)
 	topk_type = new OpaqueType("topk");
 	bloomfilter_type = new OpaqueType("bloomfilter");
 	x509_opaque_type = new OpaqueType("x509");
+	ocsp_resp_opaque_type = new OpaqueType("ocsp_resp");
 
 	// The leak-checker tends to produce some false
 	// positives (memory which had already been
@@ -881,19 +916,52 @@ int main(int argc, char** argv)
 	HeapLeakChecker::Disabler disabler;
 #endif
 
+	is_parsing = true;
 	yyparse();
+	is_parsing = false;
+
+	find_old_comm_usages();
+
+	if ( old_comm_usage_count )
+		{
+		auto old_comm_ack_id = global_scope()->Lookup("old_comm_usage_is_ok");
+
+		if ( ! old_comm_ack_id->ID_Val()->AsBool() )
+			reporter->FatalError("Detected old, deprecated communication "
+								 "system usages that will not work unless "
+								 "you explicitly take action to initizialize "
+								 "and set up the old comm. system.  "
+								 "Set the 'old_comm_usage_is_ok' flag "
+								 "to bypass this error if you've taken such "
+								 "actions, but the suggested solution is to "
+								 "port scripts to use the new Broker API.");
+		}
+
+	RecordVal::ResizeParseTimeRecords();
+
+	init_general_global_var();
+	init_net_var();
+	init_builtin_funcs_subdirs();
+
+	plugin_mgr->InitBifs();
+
+	if ( reporter->Errors() > 0 )
+		exit(1);
 
 	plugin_mgr->InitPostScript();
-	analyzer_mgr->InitPostScript();
-	file_mgr->InitPostScript();
 	broxygen_mgr->InitPostScript();
+	broker_mgr->InitPostScript();
 
 	if ( print_plugins )
 		{
-		show_plugins(print_plugins);
-		exit(1);
+		bool success = show_plugins(print_plugins);
+		exit(success ? 0 : 1);
 		}
 
+	analyzer_mgr->InitPostScript();
+	file_mgr->InitPostScript();
+	dns_mgr->InitPostScript();
+
 	if ( parse_only )
 		{
 		int rc = (reporter->Errors() > 0 ? 1 : 0);
@@ -911,9 +979,6 @@ int main(int argc, char** argv)
 		}
 
 	reporter->InitOptions();
-
-	init_general_global_var();
-
 	broxygen_mgr->GenerateDocs();
 
 	if ( user_pcap_filter )
@@ -961,8 +1026,7 @@ int main(int argc, char** argv)
 		// ### Add support for debug command file.
 		dbg_init_debugger(0);
 
-	if ( (flow_files.length() == 0 || read_files.length() == 0) &&
-	     (netflows.length() == 0 || interfaces.length() == 0) )
+	if ( read_files.length() == 0 && interfaces.length() == 0 )
 		{
 		Val* interfaces_val = internal_val("interfaces");
 		if ( interfaces_val )
@@ -977,15 +1041,8 @@ int main(int argc, char** argv)
 			}
 		}
 
-	snaplen = internal_val("snaplen")->AsCount();
-
-	// Initialize the secondary path, if it's needed.
-	secondary_path = new SecondaryPath();
-
 	if ( dns_type != DNS_PRIME )
-		net_init(interfaces, read_files, netflows, flow_files,
-			writefile, "",
-			secondary_path->Filter(), do_watchdog);
+		net_init(interfaces, read_files, writefile, do_watchdog);
 
 	BroFile::SetDefaultRotation(log_rotate_interval, log_max_size);
 
@@ -1018,23 +1075,12 @@ int main(int argc, char** argv)
 	// Just read state file from disk.
 	if ( bst_file )
 		{
-		if ( to_xml )
-			{
-			BinarySerializationFormat* b =
-				new BinarySerializationFormat();
-			XMLSerializationFormat* x = new XMLSerializationFormat();
-			ConversionSerializer s(b, x);
-			s.Convert(bst_file, "/dev/stdout");
-			}
-		else
-			{
-			FileSerializer s;
-			UnserialInfo info(&s);
-			info.print = stdout;
-			info.install_uniques = true;
-			if ( ! s.Read(&info, bst_file) )
-				reporter->Error("Failed to read events from %s\n", bst_file);
-			}
+		FileSerializer s;
+		UnserialInfo info(&s);
+		info.print = stdout;
+		info.install_uniques = true;
+		if ( ! s.Read(&info, bst_file) )
+			reporter->Error("Failed to read events from %s\n", bst_file);
 
 		exit(0);
 		}
@@ -1079,7 +1125,7 @@ int main(int argc, char** argv)
 	if ( ! reading_live && ! reading_traces )
 		// Set up network_time to track real-time, since
 		// we don't have any other source for it.
-		network_time = current_time();
+		net_update_time(current_time());
 
 	EventHandlerPtr bro_init = internal_handler("bro_init");
 	if ( bro_init )	//### this should be a function
@@ -1108,9 +1154,6 @@ int main(int argc, char** argv)
 
 	delete alive_handlers;
 
-	if ( do_notice_analysis )
-		notice_analysis();
-
 	if ( stmts )
 		{
 		stmt_flow_type flow;
@@ -1140,13 +1183,14 @@ int main(int argc, char** argv)
 	// Drain the event queue here to support the protocols framework configuring DPM
 	mgr.Drain();
 
+	broker_mgr->BroInitDone();
 	analyzer_mgr->DumpDebug();
 
 	have_pending_timers = ! reading_traces && timer_mgr->Size() > 0;
 
-	io_sources.Register(thread_mgr, true);
+	iosource_mgr->Register(thread_mgr, true);
 
-	if ( io_sources.Size() > 0 ||
+	if ( iosource_mgr->Size() > 0 ||
 	     have_pending_timers ||
 	     BifConst::exit_only_after_terminate )
 		{
@@ -1165,14 +1209,46 @@ int main(int argc, char** argv)
 
 #endif
 
+		double time_net_start = current_time(true);;
+
+		uint64 mem_net_start_total;
+		uint64 mem_net_start_malloced;
+
+		if ( time_bro )
+			{
+			get_memory_usage(&mem_net_start_total, &mem_net_start_malloced);
+
+			fprintf(stderr, "# initialization %.6f\n", time_net_start - time_start);
+
+			fprintf(stderr, "# initialization %" PRIu64 "M/%" PRIu64 "M\n",
+				mem_net_start_total / 1024 / 1024,
+				mem_net_start_malloced / 1024 / 1024);
+			}
+
 		net_run();
+
+		double time_net_done = current_time(true);;
+
+		uint64 mem_net_done_total;
+		uint64 mem_net_done_malloced;
+
+		if ( time_bro )
+			{
+			get_memory_usage(&mem_net_done_total, &mem_net_done_malloced);
+
+			fprintf(stderr, "# total time %.6f, processing %.6f\n",
+				time_net_done - time_start, time_net_done - time_net_start);
+
+			fprintf(stderr, "# total mem %" PRId64 "M/%" PRId64 "M, processing %" PRId64 "M/%" PRId64 "M\n",
+				mem_net_done_total / 1024 / 1024,
+				mem_net_done_malloced / 1024 / 1024,
+				(mem_net_done_total - mem_net_start_total) / 1024 / 1024,
+				(mem_net_done_malloced - mem_net_start_malloced) / 1024 / 1024);
+			}
+
 		done_with_network();
 		net_delete();
 
-#ifdef USE_CURL
-		curl_global_cleanup();
-#endif
-
 		terminate_bro();
 
 		sqlite3_shutdown();
